@@ -51,7 +51,7 @@ __PACKAGE__->mk_accessors(
   ),
 );
 
-our $VERSION = '0.61'; ## no critic (RequireInterpolation)
+our $VERSION = '0.62'; ## no critic (RequireInterpolation)
 
 ########################################################################
 sub new {
@@ -358,7 +358,7 @@ sub buckets {
   );
 
   return $bucket_list
-    if $self->_remember_errors($r);
+    if !$r || $self->errstr;
 
   my $owner_id          = $r->{Owner}{ID};
   my $owner_displayname = $r->{Owner}{DisplayName};
@@ -587,10 +587,18 @@ sub list_bucket {
     },
   );
 
-  return $bucket_list
-    if $self->_remember_errors($r);
+  $self->get_logger->trace(
+    Dumper(
+      [ r      => $r,
+        errstr => $self->errstr,
+      ]
+    )
+  );
 
-  $self->get_logger->debug(
+  return $bucket_list
+    if !$r || $self->errstr;
+
+  $self->get_logger->trace(
     sub {
       return Dumper(
         [ marker      => $marker,
@@ -662,6 +670,8 @@ sub list_bucket {
     $bucket_list->{common_prefixes} = \@common_prefixes;
   }
 
+  $self->get_logger->trace( Dumper( [ bucket_list => $bucket_list ] ) );
+
   return $bucket_list;
 }
 
@@ -1550,7 +1560,7 @@ managing Amazon S3 buckets and keys.
 
 =head1 DESCRIPTION
 
-This documentation refers to version 0.61.
+This documentation refers to version 0.62.
 
 C<Amazon::S3> provides a portable client interface to Amazon Simple
 Storage System (S3).
@@ -1591,7 +1601,7 @@ Perl modules in lieu of raw Perl code to increase maintainability and
 stability as well as some refactoring. C<Amazon::S3> also strives now
 to adhere to best practices as much as possible.
 
-C<Paws::S3> is a much more robust implementation of
+C<Paws::S3> may be a much more robust implementation of
 a Perl S3 interface, however this module may still appeal to
 those that favor simplicity of the interface and a lower number of
 dependencies. Below is the original description of the module.
@@ -1648,9 +1658,9 @@ dependencies are impossible to control and include XS modules.
 
 Technically, this module should run on versions 5.10 and above,
 however some of the dependencies may require higher versions of
-C<perl> or some versions of the dependencies that conflict with
-other versions of dependencies...it's a crapshoot when dealing with
-older C<perl> versions and CPAN modules.
+C<perl> or some lower versions of the dependencies due to conflicts
+with other versions of dependencies...it's a crapshoot when dealing
+with older C<perl> versions and CPAN modules.
 
 You may however, be able to build this module by installing older
 versions of those dependencies and take your chances that those older
@@ -1680,7 +1690,7 @@ Version V2.
 
 B<New regions after January 30, 2014 will only support Signature Version 4.>
 
-See L</Signature Version V4> below for important details.>
+See L</Signature Version V4> below for important details.
 
 =over 10
 
@@ -1719,9 +1729,11 @@ for S3 added after the initial creation of this interface.
 
 =item Multipart Upload Support
 
-There is some limited testing for multipart uploads.
+There are some recently added unit tests for multipart uploads that
+seem to indicate this feature is working as expected.  Please report
+any deviation from expected results if you are using those methods.
 
-For more information regarding multi-part uploads visit the link below.
+For more information regarding multipart uploads visit the link below.
 
 L<https://docs.aws.amazon.com/AmazonS3/latest/API/API_CreateMultipartUpload.html>
 
@@ -1812,8 +1824,8 @@ Defines the S3 host endpoint to use.
 default: s3.amazonaws.com
 
 Note that requests are made to domain buckets when possible.  You can
-prevent that behavior if either the bucket name does conform to DNS
-bucket naming conventions or you preface the bucket name with '/'.
+prevent that behavior if either the bucket name does not conform to
+DNS bucket naming conventions or you preface the bucket name with '/'.
 
 If you set a region then the host name will be modified accordingly if
 it is an Amazon endpoint.
@@ -1889,14 +1901,14 @@ I<blast radius> of any potential security breach.
 In this case, both the C<Amazon::S3> class and the
 L<Net::Amazon::Signature::V4> have your credentials. Caveat Emptor.
 
-See Also L<Amazon::Credentials> for more information about safely
+See also L<Amazon::Credentials> for more information about safely
 storing your credentials and preventing exfiltration.
 
 =back
 
 =head2 region
 
-Sets the region for the for the API calls. This will also be the
+Sets the region for the  API calls. This will also be the
 default when instantiating the bucket object unless you pass the
 region parameter in the C<bucket> method or use the C<verify_region>
 flag that will I<always> verify the region of the bucket using the
@@ -1931,9 +1943,8 @@ default: false
 
 =back
 
-Returns a HASHREF containing the metadata for all of the buckets
-owned by the accout or (see below) or C<undef> on
-error.
+Returns a reference to a hash containing the metadata for all of the
+buckets owned by the accout or (see below) or C<undef> on error.
 
 =over
 
@@ -1947,8 +1958,8 @@ The name of the owner account.
 
 =item buckets
 
-Any ARRAYREF of L<Amazon::S3::Bucket> objects for the 
-account.
+An array of L<Amazon::S3::Bucket> objects for the account. Returns
+C<undef> if there are not buckets or an error occurs.
 
 =back
 
@@ -2001,7 +2012,8 @@ method.
 
 =head2 delete_bucket
 
-Takes either a L<Amazon::S3::Bucket> object or a HASHREF containing:
+Takes either a L<Amazon::S3::Bucket> object or a reference to a hash
+containing:
 
 =over
 
@@ -2016,7 +2028,7 @@ determine the bucket's region by calling C<get_bucket_location>.
 
 =back
 
-Returns a boolean indicating the success of failure of the API
+Returns a boolean indicating the success or failure of the API
 call. Check C<err> or C<errstr> for error messages.
 
 Note from the L<Amazon's documentation|https://docs.aws.amazon.com/AmazonS3/latest/userguide/BucketRestrictions.html>
@@ -2046,13 +2058,13 @@ default: true
 
 List all keys in this bucket.
 
-Takes a HASHREF of arguments:
+Takes a reference to a hash of arguments:
 
 =over
 
-=item bucket
+=item bucket (required)
 
-REQUIRED. The name of the bucket you want to list keys on.
+The name of the bucket you want to list keys on.
 
 =item prefix
 
@@ -2112,27 +2124,25 @@ If C<marker> is omitted,the first page of results is returned.
 
 =back
 
-Returns C<undef> on error and a HASHREF of data on success:
+Returns C<undef> on error and a reference to a hash of data on success:
 
-The HASHREF looks like this:
+The return value looks like this:
 
   {
-        bucket       => $bucket_name,
-        prefix       => $bucket_prefix, 
-        marker       => $bucket_marker, 
-        next_marker  => $bucket_next_available_marker,
-        max_keys     => $bucket_max_keys,
-        is_truncated => $bucket_is_truncated_boolean
-        keys          => [$key1,$key2,...]
-   }
-
-Explanation of bits of that:
+   bucket       => $bucket_name,
+   prefix       => $bucket_prefix, 
+   marker       => $bucket_marker, 
+   next_marker  => $bucket_next_available_marker,
+   max_keys     => $bucket_max_keys,
+   is_truncated => $bucket_is_truncated_boolean
+   keys          => [$key1,$key2,...]
+  }
 
 =over
 
 =item is_truncated
 
-B flag that indicates whether or not all results of your query were
+Boolean flag that indicates whether or not all results of your query were
 returned in this response. If your results were truncated, you can
 make a follow-up paginated request using the Marker parameter to
 retrieve the rest of the results.
@@ -2149,17 +2159,17 @@ sent with the request.
 
 =back
 
-Each key is a HASHREF that looks like this:
+Each key is a reference to a hash that looks like this:
 
-     {
-        key           => $key,
-        last_modified => $last_mod_date,
-        etag          => $etag, # An MD5 sum of the stored content.
-        size          => $size, # Bytes
-        storage_class => $storage_class # Doc?
-        owner_id      => $owner_id,
-        owner_displayname => $owner_name
-    }
+  {
+    key           => $key,
+    last_modified => $last_mod_date,
+    etag          => $etag, # An MD5 sum of the stored content.
+    size          => $size, # Bytes
+    storage_class => $storage_class # Doc?
+    owner_id      => $owner_id,
+    owner_displayname => $owner_name
+  }
 
 =head2 get_bucket_location
 
@@ -2167,7 +2177,7 @@ Each key is a HASHREF that looks like this:
  get_bucket_locaiton(bucket-obj)
 
 This is a convenience routines for the C<get_location_constraint()> of
-the bucket object.  This method will, however return the default
+the bucket object.  This method will return the default
 region of 'us-east-1' when C<get_location_constraint()> returns a null
 value.
 
@@ -2184,7 +2194,7 @@ method of the bucket object.
 =head2 get_logger
 
 Returns the logger object. If you did not set a logger when you
-created the object then the an instance of C<Amazon::S3::Logger> is
+created the object then an instance of C<Amazon::S3::Logger> is
 returned. You can log to STDERR using this logger. For example:
 
  $s3->get_logger->debug('this is a debug message');
@@ -2197,7 +2207,7 @@ List all keys in this bucket without having to worry about
 'marker'. This is a convenience method, but may make multiple requests
 to S3 under the hood.
 
-Takes the same arguments as list_bucket.
+Takes the same arguments as C<list_bucket>.
 
 I<You are encouraged to use the newer C<list_bucket_all_v2> method.>
 
@@ -2393,13 +2403,15 @@ additional information logged at lower levels.
 
 =item L<Authenticating Requests (AWS Signature Version 2)|https://docs.aws.amazon.com/AmazonS3/latest/userguide/RESTAuthentication.html>
 
+=item L<LocalStack|https://localstack.io>
+
 =back
 
 =head1 SUPPORT
 
 Bugs should be reported via the CPAN bug tracker at
 
-<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Amazon-S3>
+L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Amazon-S3>
 
 For other issues, contact the author.
 
